// Craig S. Kaplan
// An example showing the use of a whole bunch of different 
// ControlP5 elements in a single sketch.

import controlP5.*;

ControlP5 cp5;

Button a_button;
Slider a_slider;
Knob a_knob;
RadioButton a_radio;
Textfield a_text;

void setup()
{
  size( 500, 500 );
  cp5 = new ControlP5( this );
  
  // Here's one way to add a UI element in ControlP5.
  // First use the addThingy() message (addButton in this
  // case), then issue a sequence of requests to configure
  // it.
  a_button = cp5.addButton( "A Button" );
  a_button.setSize( 100, 50 );
  a_button.setPosition( 20, 20 );
  // You can set many more properties of controls.  See all
  // the set...() methods in the online documentation for
  // the Controller class:
  // http://www.sojamo.de/libraries/controlP5/reference/index.html
  // (or don't see it -- you only really need to know what you
  // see here for Buttons).
  
  // Here's another way to add and configure a UI element 
  // all at once.  This uses a clever object-oriented 
  // programming trick.  I'm not too thrilled about explaining
  // why it works, but I definitely want you to see it 
  // so that other ControlP5 examples out in the world
  // don't look so mysterious.  The effect is the same as
  // if you configured the control one property at a time,
  // like the button above.
  a_slider = cp5.addSlider( "A Slider" )
    .setPosition( 150, 50 )
    .setSize( 25, 200 )
    .setRange( 15, 85 ) // minimum and maximum values
    .setValue( 26 ); // initial value
    
  // Just for fun, dig up a few of the lesser-used properties
  // for a knob.  Draw some tick marks, force the control to
  // snap to the nearest tick mark, and tell the knob not to
  // show any decimal digits.  Obviously, when making your
  // own control, just leave off the properties you don't care
  // about and they'll be set to reasonable defaults.
  a_knob = cp5.addKnob( "A Knob" )
    .setPosition( 250, 50 )
    .setSize( 40, 40 )
    .setRange( 1, 11 )
    .setValue( 5 )
    .setNumberOfTickMarks( 10 )
    .snapToTickMarks( true )
    .setDecimalPrecision( 0 );
    
  // Create a spot for text entry.
  a_text = cp5.addTextfield( "Say Something" )
    .setPosition( 50, 275 )
    .setSize( 200, 40 );
    
  // Configure a radio button in a kind of hybrid way -- 
  // some of the properties are done with repeated message
  // sends at the point that you create the button, but
  // the addItem() messages happen separately below.
  // NOTE: the setSize() message is a bit funny for this
  // control.  It's the size of *each* radio button, not of
  // the whole controller.
  a_radio = cp5.addRadioButton( "A Radio Button" )
    .setSize( 60, 40 ) // size of each button
    .setPosition( 50, 350 ) // position of the whole control
    .setItemsPerRow( 4 ) // adjust this when you have more or fewer buttons
    .setSpacingColumn( 50 ); // width of label space between buttons

  a_radio.addItem( "Lychee", 1 );
  a_radio.addItem( "Durian", 2 );
  a_radio.addItem( "Rambutan", 3 );
  a_radio.addItem( "Kumquat", 4 );  
  
  // This will actually cause controlEvent() to be called
  // below, even though we're still in setup.  You need to
  // be aware of that; if selecting a radio button will
  // cause something interactive to happen, you may need to
  // temporarily suppress that behaviour until you're ready
  // to deal with it.
  a_radio.activate( 2 );
}

void draw()
{
  background( 80 );
}

// The master event handler for events from user interface
// widgets.  You're passed a special object called a 
// "ControlEvent" that contains information about the
// event that just happened.  The most important information
// is: which controller did the event happen to?  Use the
// isFrom() message to ask the event to tell you.
void controlEvent( ControlEvent ev )
{
  if( ev.isFrom( a_button ) ) {
    println( "The user pressed the button" );
  } else if( ev.isFrom( a_slider ) ) {
    println( "The user set the slider to", a_slider.getValue() );
  } else if( ev.isFrom( a_knob ) ) {
    println( "The user turned the knob to", a_knob.getValue() );
  } else if( ev.isFrom( a_text ) ) {
    println( "The user typed \"" + a_text.getText() + "\"" );
  } else if( ev.isFrom( a_radio ) ) {
    println( "The user selected option", a_radio.getValue(),
      "with the radio button" );
  }
}