// Craig S. Kaplan
// Jigsaw -- a demonstration of picking objects that
// have complicated shapes, by drawing a "false colour"
// image into an off-screen image, and checking the pixels
// of that image to do hit testing.  A very effective 
// technique, because the pixels don't lie -- they
// tell you exactly where every visible object can be
// clicked.

// While the sketch is running, press the space bar to
// temporarily show the false colours used internally for
// hit testing.

int num_shapes = 0;
boolean show_false_colours = false;

// A class that holds an SVG loaded from a file, which
// can be drawn at a stored position.  It also maintains
// an internal colour that will be used for hit testing
// purposes.
class MyShape
{
  PShape piece;
  float x;
  float y;
  color col;

  MyShape( String fname )
  {
    piece = loadShape( fname );
    x = random( width );
    y = random( height );
    col = color( num_shapes * 20 + 1, 0, 0 );
    ++num_shapes;
  }

  void draw()
  {
    piece.enableStyle();
    shape( piece, x, y );
  }
  
  void drawBox()
  {
    fill( 255, 50 );
    stroke( 0, 200 );
    strokeWeight( 2 );
    rect( x, y, piece.width, piece.height );
  }

  void pickDraw( PGraphics pg )
  {
    piece.disableStyle();
    pg.noStroke();
    pg.fill( col );
    pg.shape( piece, x, y );
  }
}

MyShape[] pieces;

// Use the PGraphics class to maintain a drawable image
// off screen that's the same size as the sketch.  You don't
// need to know how to use this class yourself, but feel free
// to look at the online reference on this class.  It's very
// useful!
PGraphics picker;

// Keep track of which object we're dragging right now.
MyShape dragging;

void setup()
{
  size( 400, 600 );

  String[] names = { 
    "a.svg", "b.svg", "c.svg", "d.svg", "e.svg", "f.svg", 
    "g.svg", "h.svg", "i.svg", "j.svg", "k.svg", "l.svg" };

  // Load in the pieces.
  pieces = new MyShape[ 12 ];
  for ( int idx = 0; idx < names.length; ++idx ) {
    pieces[idx] = new MyShape( names[idx] );
  }

  picker = createGraphics( width, height );
  dragging = null;
}

void draw()
{
  // Draw the puzzle pieces (maybe using false colours
  // for illustration purposes).
  background( 128, 128, 150 );
  for ( MyShape p : pieces ) {
    if ( show_false_colours ) {
      p.pickDraw( g );
    } else {
      p.draw();
    }
  }

  // Now redraw the false colour image too, off screen.
  picker.beginDraw();
  picker.background( 0, 0, 0 );
  for ( MyShape p : pieces ) {
    p.pickDraw( picker );
  }
  picker.endDraw();
}

void mousePressed()
{
  // Here, the hit testing is surprisingly easy: read the
  // pixel from the hit test image and use that to determine
  // which object was clicked.
  
  int r = int( red( picker.get( mouseX, mouseY ) ) );
  if ( r != 0 ) {
    dragging = pieces[ (r - 1) / 20 ];
  }
}

void mouseDragged()
{
  if ( dragging != null ) {
    dragging.x += mouseX - pmouseX;
    dragging.y += mouseY - pmouseY;
  }
}

void mouseReleased()
{
  dragging = null;
}

void keyPressed()
{
  if( key == ' ' ) {
    show_false_colours = !show_false_colours;
  }
}